#ifndef MOVE_BASE_FLEX_ACTION_CLIEMT_H
#define MOVE_BASE_FLEX_ACTION_CLIEMT_H

#include <mbf_msgs/MoveBaseAction.h>
#include <mbf_msgs/GetPathAction.h>
#include <mbf_msgs/ExePathAction.h>
#include <mbf_msgs/RecoveryAction.h>
#include <ros/ros.h>
#include <actionlib/client/simple_action_client.h>

namespace mobile_platform_controller_tools {

class MBFActionClient
{
  using MoveBaseFlexClient = actionlib::SimpleActionClient<mbf_msgs::MoveBaseAction>;
  using GethPathClient = actionlib::SimpleActionClient<mbf_msgs::GetPathAction>;
  using ExePathClient = actionlib::SimpleActionClient<mbf_msgs::ExePathAction>;
  using RecoveryClient = actionlib::SimpleActionClient<mbf_msgs::RecoveryAction>;

public:
  MBFActionClient(std::string mbf_namespace);

  // std::pair<bool, std::string> driveToPose(const geometry_msgs::PoseStamped& goal);
  void sendMoveBaseGoal(const geometry_msgs::PoseStamped& goal);
  void preemptMoveBaseGoal();
  mbf_msgs::MoveBaseFeedback moveBaseFeedback();
  mbf_msgs::MoveBaseResult moveBaseResult();
  void moveBaseFeedbackCallback(const mbf_msgs::MoveBaseFeedbackConstPtr& feedback);
  bool moveBaseActionDone()
  {
    return move_base_is_done_;
  }

  void globalPlanCallback(const nav_msgs::PathConstPtr& path);
  std::pair<bool, std::string> getPath(const geometry_msgs::PoseStamped& goal);
  bool isNavServerConnected();
  void findClosestIndexGlobalPlan();
  void activate();
  void deactivate();

private:
  ros::NodeHandle private_nh_;
  MoveBaseFlexClient mbf_mb_client_;
  GethPathClient mbf_get_path_client_;
  ros::Subscriber global_plan_subscriber_;
  geometry_msgs::PoseStamped current_robot_pose_;
  nav_msgs::Path current_global_plan_;
  double supervision_rate_;
  double goal_progress_feedback_time_s_;
  std::string global_plan_topic_name_;
  double dry_run_tolerance_;
  int last_closest_pose_index_;
  double pose_reached_progress_percent_;

  // move_base
  bool move_base_is_done_;
  mbf_msgs::MoveBaseFeedback move_base_feedback_;
  mbf_msgs::MoveBaseResult move_base_result_;
  bool move_base_new_feedback_available_;
};
}  // namespace mobile_platform_controller_tools

#endif  // MOVE_BASE_FLEX_ACTION_CLIEMT_H
