#ifndef MOVE_BASE_ACTION_CLIENT_H
#define MOVE_BASE_ACTION_CLIENT_H

#include "visualization_msgs/Marker.h"
#include <actionlib/client/simple_action_client.h>
#include <move_base_msgs/MoveBaseAction.h>
#include <ros/ros.h>
#include <nav_msgs/Path.h>

namespace mobile_platform_controller_tools {

class MBActionClient
{
  using MoveBaseClient = actionlib::SimpleActionClient<move_base_msgs::MoveBaseAction>;

public:
  MBActionClient(std::string mb_namespace);

  std::pair<bool, std::string> sendMoveBaseGoal(const geometry_msgs::PoseStamped& goal);

  void moveBaseFeedbackCallback(const move_base_msgs::MoveBaseFeedbackConstPtr& feedback);
  void preemptGoal();

  void globalPlanCallback(const nav_msgs::PathConstPtr& path);
  std::pair<bool, std::string> getPath(const geometry_msgs::PoseStamped& goal);
  bool isNavServerConnected();
  void findClosestIndexGlobalPlan();

private:
  ros::NodeHandle private_nh_;
  MoveBaseClient move_base_client_;
  ros::ServiceClient move_base_service_client_;
  ros::Subscriber global_plan_subscriber_;
  geometry_msgs::PoseStamped current_robot_pose_;
  nav_msgs::Path current_global_plan_;
  double supervision_rate_;
  double goal_progress_feedback_time_s_;
  std::string global_plan_topic_name_;
  double dry_run_tolerance_;
  int last_closest_pose_index_;
  double pose_reached_progress_percent_;
  double progress_;

  // Visualize Goal
  ros::Publisher viz_mark_pub_;
  visualization_msgs::Marker current_goal_marker_;
};
} // namespace mobile_platform_controller_tools

#endif // MOVE_BASE_ACTION_CLIENT_H
