#ifndef WAYPOINTS_EXPORTER_IMPORTER_H
#define WAYPOINTS_EXPORTER_IMPORTER_H

#include <ros/ros.h>
#include <mobile_platform_controller_tools/ExportWaypoints.h>
#include <mobile_platform_controller_tools/ImportWaypoints.h>

/*
 * Advertises a service to export waypoints (vector of geoemtry_msgs::PoseStamped) as rosbag file.
 * Advertises a service to import waypoints (vector of geoemtry_msgs::PoseStamped) from rosbag file.
 */

class WaypointsExporterImporter
{
public:
  WaypointsExporterImporter(ros::NodeHandle & private_nh);

  bool exportWaypoints(mobile_platform_controller_tools::ExportWaypoints::Request& req,
                       mobile_platform_controller_tools::ExportWaypoints::Response& res);
  bool importWaypoints(mobile_platform_controller_tools::ImportWaypoints::Request& req,
                       mobile_platform_controller_tools::ImportWaypoints::Response& res);
  std::string amendFileName(const std::string& str);
  bool readPosesFromTextFile(const std::string& poses_file,
                             std::vector<geometry_msgs::PoseStamped>& g_poses);

private:
  ros::NodeHandle private_handle_;
  ros::ServiceServer export_waypoints_server_;
  ros::ServiceServer import_waypoints_server_;
  std::string exported_topic_name_;  // used as topic name when exporting as rosbag file.
  bool export_as_txt_;  // force txt format independently of request file name extension.
};

#endif  // WAYPOINTS_EXPORTER_IMPORTER_H
